'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var fs = require('fs');
var inplace = _interopDefault(require('json-in-place'));
var path = require('path');
var parser = _interopDefault(require('yargs-parser'));

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var NAME = 'jest-ratchet';
function getLastError(config) {
    var errors = [];
    var collectCoverage = config.collectCoverage, coverageReporters = config.coverageReporters;
    if ((coverageReporters || []).indexOf('json-summary') === -1) {
        errors.push("'json-summary' needs to be listed as a coverageReporter in order for " + NAME + " to work appropriately.");
    }
    if (!collectCoverage) {
        errors.push("'collectCoverage' option needs to be enabled in order for " + NAME + " to work appropriately.");
    }
    if (errors.length > 0) {
        throw new Error(errors.join('\n'));
    }
}

function updateFile(fileName, result) {
    var jestConfigRaw = fs.readFileSync(fileName, 'utf-8');
    var jestConfig = JSON.parse(jestConfigRaw);
    var prefix = jestConfig.jest ? 'jest.' : '';
    var newFile = setCoverage(jestConfigRaw, result, prefix);
    fs.writeFileSync(fileName, newFile, 'utf-8');
}
function setCoverage(source, result, prefix) {
    prefix += 'coverageThreshold.';
    var newSource = inplace(source);
    for (var _i = 0, _a = Object.keys(result); _i < _a.length; _i++) {
        var key = _a[_i];
        if (result[key].branches) {
            newSource.set(prefix + key + '.branches', result[key].branches);
        }
        if (result[key].functions) {
            newSource.set(prefix + key + '.functions', result[key].functions);
        }
        if (result[key].lines) {
            newSource.set(prefix + key + '.lines', result[key].lines);
        }
        if (result[key].statements) {
            newSource.set(prefix + key + '.statements', result[key].statements);
        }
    }
    return newSource.toString();
}

function findJestConfigPath(cwd, argv) {
    var configLocation = 'package.json';
    var args = parser(argv.slice(2));
    if (args && args.config) {
        configLocation = args.config;
    }
    if (!path.isAbsolute(configLocation)) {
        configLocation = path.resolve(cwd, configLocation);
    }
    return configLocation;
}
function findCoveragePath(config) {
    if (config.coverageDirectory) {
        return config.coverageDirectory;
    }
    if (config.rootDir) {
        return path.resolve(config.rootDir, 'coverage');
    }
    return path.resolve(process.cwd(), 'coverage');
}
function findCoverageSummaryPath(coverageDirectory) {
    return path.resolve(coverageDirectory, 'coverage-summary.json');
}

function noop() {
}

function ratchetCoverage(threshold, summary, options) {
    var result = {};
    for (var _i = 0, _a = Object.keys(threshold); _i < _a.length; _i++) {
        var key = _a[_i];
        var summaryKey = key === 'global' ? 'total' : key;
        result[key] = ratchetSingleCoverage(threshold[key], summary[summaryKey], options);
    }
    return result;
}
function ratchetSingleCoverage(threshold, summary, options) {
    var branches = threshold.branches, functions = threshold.functions, lines = threshold.lines, statements = threshold.statements;
    return {
        branches: ratchetSingleNumberCoverage(branches, summary.branches, options),
        functions: ratchetSingleNumberCoverage(functions, summary.functions, options),
        lines: ratchetSingleNumberCoverage(lines, summary.lines, options),
        statements: ratchetSingleNumberCoverage(statements, summary.statements, options),
    };
}
function ratchetSingleNumberCoverage(num, category, options) {
    if (num && category) {
        var ratchetPct = options.ratchetPercentagePadding
            ? Math.round(category.pct) - options.ratchetPercentagePadding
            : category.pct;
        if (num > 0 && num <= ratchetPct) {
            return ratchetPct;
        }
        else if (num < 0 && num >= -category.covered) {
            return -category.covered;
        }
    }
}

var JestRatchet = (function () {
    function JestRatchet(globalConfig, options) {
        if (options === void 0) { options = {}; }
        this.getLastError = noop;
        this.onRunComplete = noop;
        if (!process.env.DISABLE_JEST_RATCHET) {
            this.onRunComplete = onRunComplete.bind(this, globalConfig, options);
            this.getLastError = getLastError.bind(this, globalConfig);
        }
    }
    return JestRatchet;
}());
function onRunComplete(globalConfig, options) {
    options = options;
    try {
        var coverageDirectory = findCoveragePath(globalConfig);
        var coverageSummaryPath_1 = findCoverageSummaryPath(coverageDirectory);
        var jestConfigPath_1 = findJestConfigPath(process.cwd(), process.argv);
        if (!fs.existsSync(coverageDirectory)) {
            fs.mkdirSync(coverageDirectory);
        }
        if (!fs.existsSync(coverageSummaryPath_1)) {
            fs.closeSync(fs.openSync(coverageSummaryPath_1, 'w'));
        }
        var watcher_1 = fs.watch(coverageDirectory);
        watcher_1.once('change', function () {
            watcher_1.close();
            var coverageRaw = fs.readFileSync(coverageSummaryPath_1, 'utf-8');
            var summary = JSON.parse(coverageRaw);
            var threshold = globalConfig.coverageThreshold;
            var ratchetResult = ratchetCoverage(threshold, summary, options);
            updateFile(jestConfigPath_1, ratchetResult);
        });
    }
    catch (e) {
        console.error(e);
    }
}

module.exports = JestRatchet;
